// B471CntSample.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#define OnExit(ErrCode)\
{\
	printf( "\n\nDevice deinitialization................................................... " );\
	int iRes = Deinitialize();\
	if( iRes != NO_ERRORS ) printf( "error code %d", iRes ); else printf( "done" );\
	printf("\n\nPress ENTER to exit");\
	getchar();\
	return ErrCode;\
}\

int main(int argc, char* argv[])
{
	int iRes = NO_ERRORS;

//////////////////////////////////////////////////////////////////////////
	printf( "\nB471 ""TiMes"" Electronic Counter Sample Application v.2.2" );
	printf( "\n(C) Alexander A. Shanditsev, AURIS, 10/06/2010." );

//////////////////////////////////////////////////////////////////////////
	printf( "\n\nTrying to find and initialize the B471 device............................. " );

	iRes = Initialize( 0x0471, 0x6871, 0xFFFF );
	if( iRes != NO_ERRORS ) { printf( "error code %d", iRes ); OnExit(-1); } else printf( "done" );

	B471CntChanSettings ch0, ch1;
	if( iRes == NO_ERRORS ) iRes = GetChanSettings( 0, &ch0 );
	if( iRes == NO_ERRORS ) iRes = GetChanSettings( 0, &ch1 );

	Sleep(2000);
//////////////////////////////////////////////////////////////////////////
	printf( "\n\nConnecting measuring channels to calibrator............................... " );

	ch0.bDivider = FALSE;		// TRUE/FALSE = "1:10"/"1:1" (on/off)
	ch0.bInput = FALSE;			// TRUE/FALSE = input/calibrator
	ch0.ssStartCh = ssCH0;		// enum B471SignalSource
	ch0.ssEventCh = ssCH0;		// --//--
	ch0.ssStartSlope = ssRISE;	// enum B471SignalSlope
	ch0.ssEventSlope = ssRISE;	// --//--
	ch0.lmLedMode = lmBOTH;	// enum B471LedMode
	ch0.dwEventsLimit = 5;		// number of events to measure

	ch1.bDivider = FALSE;		// TRUE/FALSE = "1:10"/"1:1" (on/off)
	ch1.bInput = FALSE;			// TRUE/FALSE = input/calibrator
	ch1.ssStartCh = ssCH1;		// enum B471SignalSource
	ch1.ssEventCh = ssCH1;		// --//--
	ch1.ssStartSlope = ssRISE;	// enum B471SignalSlope
	ch1.ssEventSlope = ssFALL;	// --//--
	ch1.lmLedMode = lmBOTH;	// enum B471LedMode
	ch1.dwEventsLimit = 5;		// number of events to measure

	if( iRes == NO_ERRORS ) iRes = SetChanSettings( 0, ch0 );
	if( iRes == NO_ERRORS ) iRes = SetChanSettings( 1, ch1 );
//	dev.dwRetriesLimit = 1;		// communication faults limit

	if( iRes == NO_ERRORS ) iRes = SetDiskrLevel( 0, 1.0 );	// Volts * (divider attenuation ratio)
	if( iRes == NO_ERRORS ) iRes = SetDiskrLevel( 1, 1.0 );	// Volts * (divider attenuation ratio)
	
	if( iRes ) { printf( "error code %d", iRes ); OnExit(-1); } else printf( "done" );

	Sleep(1000);
//////////////////////////////////////////////////////////////////////////
	printf( "\n\nStarting measurement...................................................... " );

	iRes = StartCh(-1);
	if( iRes != NO_ERRORS ) { printf( "error code %d", iRes ); OnExit(-1); } else printf( "done" );

	Sleep(1000);
//////////////////////////////////////////////////////////////////////////
	printf( "\n\nConfiguring calibrator (still no events, leds are off).................... " );

	B471ClbrSettings Clbr;
	iRes = GetClbrSettings( &Clbr );
	if( iRes == NO_ERRORS )
	{
		Clbr.bActiveHi = TRUE;
		Clbr.bEnabled = FALSE;
		Clbr.fPeriod = 1.0;		// 1s
		Clbr.fLength = 0.2;		// 200ms
	}
	if( iRes == NO_ERRORS ) iRes = SetClbrSettings( Clbr );

	if( iRes == NO_ERRORS ) iRes = SetClbrLevel( TRUE,  2.0 );
	if( iRes == NO_ERRORS ) iRes = SetClbrLevel( FALSE, 0.0 );

	if( iRes != NO_ERRORS ) { printf( "error code %d", iRes ); OnExit(-1); } else printf( "done" );

//////////////////////////////////////////////////////////////////////////
	Sleep(2000);
	printf( "\n\nEnabling calibrator output, leds spark 10 times (0,2s - 0.8s - 0.2s)...... " );

	Clbr.bEnabled = TRUE;

	if( iRes == NO_ERRORS ) iRes = SetClbrSettings( Clbr );

	if( iRes != NO_ERRORS ) { printf( "error code %d", iRes ); OnExit(-1); } else printf( "done" );

//////////////////////////////////////////////////////////////////////////
	printf( "\n\nWaiting 10 sec (max) for measuring is complete (5 events)" );

	double ClkTime = 10e-9;
	if( iRes == NO_ERRORS ) iRes = GetTimeBase( &ClkTime ); // =2.5e-9 for v.2.0
	
	DWORD dwTime = 0, dwEvents[2];
	B471CntChanStatus st0, st1;
	st0.bMeasEn = st1.bMeasEn = TRUE;
	DWORD *pData0 = NULL, *pData1 = NULL;

	while ((dwTime++ < 10) && (st0.bMeasEn | st1.bMeasEn))
	{
		Sleep(1000);
		if( iRes == NO_ERRORS ) iRes = GetStatusCh( 0, &st0 );
		if( iRes == NO_ERRORS ) iRes = GetStatusCh( 1, &st1 );
		if( iRes == NO_ERRORS ) iRes = GetDataCh( 0, 0, &pData0 );
		if( iRes == NO_ERRORS ) iRes = GetDataCh( 1, 0, &pData1 );
		
		if( iRes != NO_ERRORS ) { printf( "error code %d", iRes ); OnExit(-1); }

		dwEvents[0] = st0.dwDataSize + st0.dwFIFOEvents; if( dwEvents[0] ) dwEvents[0]--;
		dwEvents[1] = st1.dwDataSize + st1.dwFIFOEvents; if( dwEvents[1] ) dwEvents[1]--;
		printf( "\n%d: Ch1 - %d / Ch2 - %d", dwTime, dwEvents[0], dwEvents[1] );

		// uncomment these lines if you want to see new events at real-time
		// if (st0.dwDataSize > 1 || st1.dwDataSize > 1)
		// {
		// 	for (DWORD i = 1; i < min(st0.dwDataSize, st1.dwDataSize); i++) // Without start event (leading zero)
		// 		printf( "\n%.10f\t%.10f", (double)pData0[i] * ClkTime, (double)pData1[i] * ClkTime );
		// 	if (st1.dwDataSize < st0.dwDataSize)
		// 		for (i = st1.dwDataSize; i < st0.dwDataSize; i++)
		// 			printf( "\n%.10f\t------------", (double)pData0[i] * ClkTime );
		// 	if (st0.dwDataSize < st1.dwDataSize)
		// 		for (i = st0.dwDataSize; i < st1.dwDataSize; i++)
		// 			printf( "\n------------\t%.10f", (double)pData1[i] * ClkTime );
		// 	printf("\n");
		// }
	}

	if( dwTime == 10 ) { printf( "\nTimeout." ); OnExit(-1); }

//////////////////////////////////////////////////////////////////////////
	printf( "\n\nReading measurement data and creating report ............................. " );

	char DateStart0[9], TimeStart0[9], DateStop0[9], TimeStop0[9];
	if( iRes == NO_ERRORS ) iRes = GetStartTime( 0, DateStart0, TimeStart0 );
	if( iRes == NO_ERRORS ) iRes = GetStopTime( 0, DateStop0, TimeStop0 );

	char DateStart1[9], TimeStart1[9], DateStop1[9], TimeStop1[9];
	if( iRes == NO_ERRORS ) iRes = GetStartTime( 1, DateStart1, TimeStart1 );
	if( iRes == NO_ERRORS ) iRes = GetStopTime( 1, DateStop1, TimeStop1 );

	if ( iRes != NO_ERRORS ) { printf( "error code %d", iRes ); OnExit(-1); }

	printf("\n\nB471 (GIVI-TiMeS) report. (c)AURIS, 2005-2007.\nCH1: started %s %s, stopped %s %s\nCH2: started %s %s, stopped %s %s\n\n###\tCH1_data\tCH2_data\n", \
		DateStart0, TimeStart0, DateStop0, TimeStop0, DateStart1, TimeStart1, DateStop1, TimeStop1);
	
	if( iRes == NO_ERRORS ) iRes = GetStatusCh( 0, &st0 );
	if( iRes == NO_ERRORS ) iRes = GetStatusCh( 1, &st1 );
	if( iRes == NO_ERRORS ) iRes = GetDataCh( 0, 0, &pData0 );
	if( iRes == NO_ERRORS ) iRes = GetDataCh( 1, 0, &pData1 );

	if ( iRes != NO_ERRORS || !pData0 || !pData1 ) { printf( "error code %d", iRes ); OnExit(-1); }

	for (DWORD i = 1; i < min(st0.dwDataSize, st1.dwDataSize); i++) // Without start event (leading zero)
		printf( "%d\t%.10f\t%.10f\n", i, (double)pData0[i] * ClkTime, (double)pData1[i] * ClkTime );

	if (st1.dwDataSize < st0.dwDataSize)
		for (i = st1.dwDataSize; i < st0.dwDataSize; i++)
			printf( "%d\t%.10f\t------------\n", i, (double)pData0[i] * ClkTime );

	if (st0.dwDataSize < st1.dwDataSize)
		for (i = st0.dwDataSize; i < st1.dwDataSize; i++)
			printf( "%d\t------------\t%.10f\n", i, (double)pData1[i] * ClkTime );

	OnExit(0);
}
