function [ ] = B471CntSample( )

clc; clear;

%% DLL Loading
if ~libisloaded( 'B471Drv' ) loadlibrary( 'B471Drv', 'B471Drv.h '); end;
if ~libisloaded( 'B471Drv' ) disp( 'B471Drv.dll is not loaded' ); return; end;

%% Functions and parameters list
libfunctions('B471Drv', '-full')

%% Simple DLL Test
if ( calllib( 'B471Drv', 'Test' ) ~= 2 ) disp('B471Drv.dll Test failed'); return; end;

%% Device Initialization
iErr = calllib('B471Drv', 'Initialize', hex2dec('0471'), hex2dec('6871'), hex2dec('FFFF') );
if (iErr ~= 0)
    iRes = calllib('B471Drv', 'Deinitialize');
    iErr = calllib('B471Drv', 'Initialize', hex2dec('0471'), hex2dec('6871'), hex2dec('FFFF') );
end;
if (iErr == 0) disp( 'Device init done' ); else disp( sprintf('Device init failed (%d)', iErr) ); return; end;

%% Getting Device Info (for example)
Ser = int32(0); pSer = libpointer( 'int32Ptr', Ser );
[ iErr, Ser ] = calllib( 'B471Drv', 'GetDevId', pSer );
clear pSer;

%% Getting Measuring Channels Names
czNameCh0 = repmat( ' ', [1,16] ); czNameCh1 = czNameCh0;
[ iErr, czNameCh0 ] = calllib( 'B471Drv', 'GetChanName', 0, czNameCh0 );
[ iErr, czNameCh1 ] = calllib( 'B471Drv', 'GetChanName', 1, czNameCh1 );

%% Setting up Channels
ch0 = InitChan( ); pCh0 = libpointer( 'B471CntChanSettingsPtr', ch0 );
iErr = calllib( 'B471Drv', 'GetChanSettings', 0, pCh0 );  ch0 = get( pCh0, 'value' );
ch1 = InitChan( ); pCh1 = libpointer( 'B471CntChanSettingsPtr', ch1 );
iErr = calllib( 'B471Drv', 'GetChanSettings', 1, pCh1 );  ch1 = get( pCh1, 'value' );
clear pCh0 pCh1;

ch0.bDivider = 0;       % 1/0 = "1:10"/"1:1" (on/off)
ch0.bInput = 0;         % 1/0 = input/calibrator
ch0.ssStartCh = 0;      % ->I (B471SignalSource)
ch0.ssEventCh = 0;      % ->I (B471SignalSource)
ch0.ssStartSlope = 1;   % ssRISE (B471SignalSlope)
ch0.ssEventSlope = 1;	% ssRISE (B471SignalSlope)
ch0.lmLedMode = 6;      % lmRISE (B471LedMode)
ch0.dwEventsLimit = 5;  % number of events to measure

ch1.bDivider = 0;       % 1/0 = "1:10"/"1:1" (on/off)
ch1.bInput = 0;         % 1/0 = input/calibrator
ch1.ssStartCh = 1;      % ->II (B471SignalSource)
ch1.ssEventCh = 1;      % ->II (B471SignalSource)
ch1.ssStartSlope = 1;   % ssRISE (B471SignalSlope)
ch1.ssEventSlope = 0;	% ssFALL (B471SignalSlope)
ch1.lmLedMode = 5;      % lmFALL (B471LedMode)
ch1.dwEventsLimit = 5;  % number of events to measure

iErr = calllib( 'B471Drv', 'SetChanSettings', 0, ch0 );
iErr = calllib( 'B471Drv', 'SetChanSettings', 1, ch1 );
iErr = calllib( 'B471Drv', 'SetDiskrLevel', 0, 1.0 );
iErr = calllib( 'B471Drv', 'SetDiskrLevel', 1, 1.0 );

%% Setting up Calibrator
Clbr = InitClbr( );  pClbr = libpointer( 'B471ClbrSettingsPtr', Clbr );
[ iErr, Clbr ] = calllib( 'B471Drv', 'GetClbrSettings', pClbr );
clear pClbr;

Clbr.fPeriod = 1.0; % 1.0 sec
Clbr.fLength = 0.2; % 0.2 sec
Clbr.bActiveHi = 1; % ActiveHI
Clbr.bEnabled = 0;  % OFF
Clbr.lmLedMode = 7; % lmBOTH (B471LedMode)
iErr = calllib( 'B471Drv', 'SetClbrSettings', Clbr );
iErr = calllib( 'B471Drv', 'SetClbrLevel', 1, 2.0 );
iErr = calllib( 'B471Drv', 'SetClbrLevel', 0, 0.0 );

%% Setting up Trigger Output
% Trig = InitTrig( );  pTrig = libpointer( 'B471TrigSettingsPtr', Trig );
% [ iErr, Trig ] = calllib( 'B471Drv', 'GetTrigSettings', pTrig );
% clear pTrig;
% and here you can change TrigOut parameters (the same as for calibrator)

%% Starting Measurement
iErr = calllib( 'B471Drv', 'StartCh', -1 );

%% Enabling Calibrator (start and stop events are coming)
Clbr.bEnabled = 1;
iErr = calllib( 'B471Drv', 'SetClbrSettings', Clbr );

%% Waiting 10s for measuring is complete
st0 = InitStatus( ); pSt0 = libpointer( 'B471CntChanStatusPtr', st0 );
st1 = InitStatus( ); pSt1 = libpointer( 'B471CntChanStatusPtr', st1 );
dwCnt = uint32(10); dwEvents0 = uint32(0); dwEvents1 = uint32(0);
while (dwCnt > 0)
    !Sleep.exe 500
    iErr = calllib( 'B471Drv', 'GetStatusCh', 0, pSt0 );   st0 = get( pSt0, 'value' );
    iErr = calllib( 'B471Drv', 'GetStatusCh', 1, pSt1 );   st1 = get( pSt1, 'value' );
    if ( st0.bMeasEn == 0 && st1.bMeasEn == 0), break; end;
    dwEvents0 = st0.dwDataSize + st0.dwFIFOEvents;
    dwEvents1 = st1.dwDataSize + st1.dwFIFOEvents;
    if dwEvents0 > 0, dwEvents0 = dwEvents0 - 1; end; % Zero time event (start) compensation
    if dwEvents1 > 0, dwEvents1 = dwEvents1 - 1; end; % Zero time event (start) compensation
    disp( sprintf( 'Ch1 - %d / Ch2 - %d', dwEvents0, dwEvents1 ) );
    dwCnt = dwCnt - 1;
end;
% Advanced measurement fault info presented by
% st_.bFIFOError  st_.bReadFail  st_.bTimeOut

if dwCnt > 0
    iErr = calllib( 'B471Drv', 'GetStatusCh', 0, pSt0 );   st0 = get( pSt0, 'value' );
    iErr = calllib( 'B471Drv', 'GetStatusCh', 1, pSt1 );   st1 = get( pSt1, 'value' );
    dwEvents0 = st0.dwDataSize; % + st0.dwFIFOEvents;
    dwEvents1 = st1.dwDataSize; % + st1.dwFIFOEvents;
    if dwEvents0 > 0, dwEvents0 = dwEvents0 - 1; end; % Zero time event (start) compensation
    if dwEvents1 > 0, dwEvents1 = dwEvents1 - 1; end; % Zero time event (start) compensation
    disp( sprintf( 'READ DONE:\nCh1 - %d / Ch2 - %d', dwEvents0, dwEvents1 ) );
else
    disp( 'Timeout' );
end
clear pSt0 pSt1;

TimeBase = double(0); pTB = libpointer( 'doublePtr', TimeBase );
[ iErr, TimeBase ] = calllib( 'B471Drv', 'GetTimeBase', pTB );
clear pTB;

if st0.dwDataSize > 0
    data0 = zeros( [st0.dwDataSize, 1], 'uint32' );
    pDt0 = libpointer( 'uint32Ptr' , data0 );
    iErr = calllib( 'B471Drv', 'FillDataCh' , 0, pDt0 );
    data0 = get( pDt0, 'value' );
    EventsCh0 = double(data0(2:end)) * TimeBase
    clear pDt0;
end
if st1.dwDataSize > 0
    data1 = zeros( [st1.dwDataSize, 1], 'uint32' );
    pDt1 = libpointer( 'uint32Ptr' , data1 );
    iErr = calllib( 'B471Drv', 'FillDataCh' , 1, pDt1 );
    data1 = get( pDt1, 'value' );
    EventsCh1 = double(data1(2:end)) * TimeBase
    clear pDt1;
end

%% Device Deinitialization
iRes = calllib('B471Drv', 'Deinitialize');
if (iRes == 0) disp( 'Device deinit done' ); else disp( sprintf('Device deinit failed (%d)', iRes) ); return; end;

%% DLL Unloading
unloadlibrary 'B471Drv'
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ ch ] = InitChan( )
    ch = libstruct( 'B471CntChanSettings' );
    ch.bDivider = int32(0);
	ch.bInput = int32(0);
	ch.ssStartCh = uint32(0);
	ch.ssStartSlope = uint32(0);
	ch.ssEventCh = uint32(0);
	ch.ssEventSlope = uint32(0);
	ch.dwEventsLimit = uint32(0);
	ch.dwTimeLimit = uint32(0);
    ch.lmLedMode = uint32(0);
end

function [ clbr ] = InitClbr( )
    clbr = libstruct( 'B471ClbrSettings' );
    clbr.bEnabled   = int32(0);
	clbr.dwEnMaskCh = uint32(0);
	clbr.bActiveHi  = int32(0);
	clbr.fPeriod    = double(0);
	clbr.fLength    = double(0);
    clbr.lmLedMode  = uint32(0);
end

function [ trig ] = InitTrig( )
    trig = libstruct( 'B471TrigSettings' );
    trig.bEnabled   = int32(0);
	trig.dwEnMaskCh = uint32(0);
	trig.bActiveHi  = int32(0);
	trig.fPeriod    = double(0);
	trig.fLength    = double(0);
    trig.lmLedMode  = uint32(0);
end

function [ st ] = InitStatus( )
    st = libstruct( 'B471CntChanStatus' );
    st.bStopped = int32(0);
    st.bMeasEn = int32(0);
    st.bBusy = int32(0);
    st.dwDataSize = uint32(0);
    st.dwFIFOSize = uint32(0);
    st.dwFIFOEvents = uint32(0);
    st.bFIFOError = int32(0);
    st.bReadFail = int32(0);
    st.dwTime = uint32(0);
    st.bTimeOut = int32(0);
    st.bLedOn = int32(0);
end
