#if !defined(AFX_B471CNTDEVICE_H__415CFF8B_104F_43E7_BD75_1721B2FFBCDA__INCLUDED_)
#define AFX_B471CNTDEVICE_H__415CFF8B_104F_43E7_BD75_1721B2FFBCDA__INCLUDED_

//////////////////////////////////////////////////////////////////////////
// Used Constants
#define B471_CHAN_NUM 2
#define B471_CHAN_NAME_LENGTH 16

// Error codes
const int NO_ERRORS		=  0;	// OK
const int ERR_NODEVICE	= -1;	// device not connected / not initialized
const int ERR_CH		= -2;	// invalid channel index
const int ERR_RANGE		= -3;	// requested dwCode is out of range
const int ERR_PARAM		= -4;	// one of other func params is out of range
const int ERR_FAILED	= -5;	// device control/data transfer failed
const int ERR_TMPFILE0	= -10;	// TMP file create error ( file already open )
const int ERR_TMPFILE1	= -11;	// TMP file create error ( disk full )
const int ERR_TMPFILE2	= -13;	// TMP file size error ( not enough disk space )
const int ERR_TMPFILE3	= -13;	// TMP file mapping creation failed
const int ERR_TMPFILE4	= -14;	// TMP file map view failed

//enum B471LedMode
const unsigned long lmOFF  = 0; // led is always off
const unsigned long lmLO   = 1; // led is on when signal under discriminator threshold level
const unsigned long lmHI   = 2; // led is on when signal above discriminator threshold level
const unsigned long lmON   = 3; // led is always on (for ex., led test)
//const int lmOFF_ = 4;// led is always off
const unsigned long lmFALL = 5; // led sparks when signal falls under discriminator threshold level
const unsigned long lmRISE = 6; // led sparks when signal rises above discriminator threshold level
const unsigned long lmBOTH = 7; // led sparks on both signal edges (lmFALL + lmRISE)

//enum B471SignalSource
const unsigned long ssCH0 = 0; // Signal input I is connected to measuring channel
const unsigned long ssCH1 = 1; // Signal input II is connected to measuring channel
const unsigned long ssTI  = 2; // Trigger input is connected to measuring channel (*not implemented, reserved)
const unsigned long ssTO  = 3; // Trigger output is connected to measuring channel (*not implemented, reserved)

//enum B471SignalSlope
const unsigned long ssFALL = 0; // Falling signal slope is used as "start" or "event"
const unsigned long ssRISE = 1; // Rising signal slope is used as "start" or "event"


//////////////////////////////////////////////////////////////////////////
// B471CntDevSettings
typedef struct {
	unsigned long	lmLedMode;		// LED mode (0/1/../7 - lmOFF/lmLO/lmHI/lmON/lmOFF_/lmFALL/lmRISE/lmBOTH)
	long			bEnabled;		// 0/1 - OFF/ON
	unsigned long	dwEnMaskCh;		// reserved (=0)
	long			bActiveHi;		// fLength defines 0/1 - active lo/hi pulse length
	double			fPeriod;		// pulse period
	double			fLength;		// pulse length
} B471TrigSettings;

typedef struct {
	unsigned long	lmLedMode;		// LED mode (0/1/../7 - lmOFF/lmLO/lmHI/lmON/lmOFF_/lmFALL/lmRISE/lmBOTH)
	long			bEnabled;		// 0/1 - OFF/ON
	unsigned long	dwEnMaskCh;		// reserved (=0)
	long			bActiveHi;		// fLength defines 0/1 - active lo/hi pulse length
	double			fPeriod;		// pulse period
	double			fLength;		// pulse length
} B471ClbrSettings;

typedef struct {
	unsigned long	lmLedMode;		// LED mode (0/1/../7 - lmOFF/lmLO/lmHI/lmON/lmOFF_/lmFALL/lmRISE/lmBOTH)
	long			bDivider;		// attenuation ratio (0/1 - 1:1/1:10)
	long			bInput;			// measuring channel signal source (0/1 - Clbr/Input)
	unsigned long	ssStartCh;		// Start signal (0/1/2/3 - ssCHI/ssCHII/ssTI/ssTO)
	unsigned long	ssStartSlope;	// Start signal slope (0/1 - ssFALL/ssRISE)
	unsigned long	ssEventCh;		// Events signal (0/1/2/3 - ssCHI/ssCHII/ssTI/ssTO)
	unsigned long	ssEventSlope;	// Event signal slope (0/1 - ssFALL/ssRISE)
	unsigned long	dwEventsLimit;	// Events Count - end of measurement condition
	unsigned long	dwTimeLimit;	// in milliseconds (by computer clock) - end of measurement condition
} B471CntChanSettings;

//////////////////////////////////////////////////////////////////////////
//
typedef struct {
	long			bStopped;		// program flag (for threads syncronization)
	long			bMeasEn;		// device flag (channel is waiting for start event or is in measuring cycle)
	long			bBusy;			// device flag (channel in measuring cycle)
	unsigned long	dwDataSize;		// Channel's events count, which are already transmitted from device
	unsigned long	dwFIFOSize;		// defines FIFO size in EVENTS (depends on PLD configuration)
	unsigned long	dwFIFOEvents;	// device FIFO status (measured events, ready for read)
	long			bFIFOError;		// device flag (indicates there was FIFO FULL, measuring cycle stops immediately to prevent events loss)
	long			bReadFail;		// program flag (indicates there was at least one error during GetData(), for ex.: if requested 12, but read 11 events)
	unsigned long	dwTime;			// program controlled (indicates time in milliseconds from measurement start, not start event(!) )
	long			bTimeOut;		// program flag (indicates channel was stopped on timeout)
	long			bLedOn;			// device flag (indicates whether channel LED is on or off)
} B471CntChanStatus;

//////////////////////////////////////////////////////////////////////////
// Functions list

// test function (always returns 2)
extern "C" int Test ();

// work begin/end functions
extern "C" int Initialize(unsigned short pid, unsigned short vid, unsigned short ser);
extern "C" int Reset();
extern "C" int Deinitialize();

// info functions
extern "C" int GetDevId( int* piSerNo );				// serial number of connected device
extern "C" int GetChanName( int ch, char* pChName );	// front panel channel name

// basic device settings functions
extern "C" int GetChanSettings( int ch, B471CntChanSettings* pChSet );	// first-time-call returns default channel settings
extern "C" int SetChanSettings( int ch, B471CntChanSettings ChSet );
extern "C" int GetDiskrLevel( int ch, float* pfValue);
extern "C" int SetDiskrLevel( int ch, float fValue);

extern "C" int GetClbrSettings( B471ClbrSettings* pClbrSet );	// first-time-call returns default channel settings
extern "C" int SetClbrSettings( B471ClbrSettings ClbrSet );
extern "C" int GetClbrLevel( long bHiLevel, float* pfValue );
extern "C" int SetClbrLevel( long bHiLevel, float fValue );		// sets given calibrator level immediately

extern "C" int GetTrigSettings( B471TrigSettings* pTrigSet );	// first-time-call returns default channel settings
extern "C" int SetTrigSettings( B471TrigSettings TrigSet );

extern "C" int StartCh( int ch );	// Starts measuring channel(s) with current configuration
extern "C" int StopCh( int ch );	// Stops measuring channel(s)

extern "C" int GetStatusCh( int ch, B471CntChanStatus* pStatusCh );

// advanced device settings functions
extern "C" int GetRetriesLimit( unsigned long* pdwRetriesLimit );
extern "C" int SetRetriesLimit( unsigned long dwRetriesLimit );

extern "C" int SetClkPpm( float fValue );
extern "C" int GetClkPpm( float* pfValue );
extern "C" int SetClkCode( unsigned long dwCode );
extern "C" int GetClkCode( unsigned long* pdwCode );
extern "C" int GetClkCodeByValue( float fValue, unsigned long* pdwCode );
extern "C" int GetClkPpmByCode(unsigned long dwCode, float* pfValue);

extern "C" int SetDiskrCode( int ch, unsigned long dwCode );
extern "C" int GetDiskrCode( int ch, unsigned long* pdwCode );
extern "C" int GetDiskrCodeByValue( int ch, float fValue, unsigned long* pdwCode );
extern "C" int GetDiskrLevelByCode( int ch, unsigned long dwCode, float* pfValue );

extern "C" int SetClbrCode( long bHiLevel, unsigned long dwCode );
extern "C" int GetClbrCode( long bHiLevel, unsigned long* pdwCode );
extern "C" int GetClbrCodeByValue( long bHiLevel, float fValue, unsigned long* pdwCode );
extern "C" int GetClbrLevelByCode( long bHiLevel, unsigned long dwCode, float* pfValue );

extern "C" int GetStartTime( int ch, char* czDateStart, char* czTimeStart );
extern "C" int GetStopTime( int ch, char* czDateStop, char* czTimeStop );

// gettings results
extern "C" int GetDataCh(int ch, unsigned long dwIndex, unsigned long** ppData );
extern "C" int FillDataCh(int ch, unsigned long* pData);
extern "C" int GetTimeBase( double* pfTimeBase );

#endif // !defined(AFX_B471CNTDEVICE_H__415CFF8B_104F_43E7_BD75_1721B2FFBCDA__INCLUDED_)
